Columns
Items yielded by a matrix provider. Used to iterate over when computing what they look like w.r.t. the latest basis.
Modules
- identity
- Identity column
Structs
- Dense
Slice Iterator - Sparse
Column - Wrapping a sparse vector of tuples.
- Sparse
Option Iterator - Sparse
Slice Iterator
Traits
- Column
- Columns represent part of a (virtual) data matrix.
- Column
Iterator - Iterator for a column.
Trait Aliases
- Column
Into Iterator - Column
Number - Basic operations that should be possible with the type of the values of the column.