Module fully

Module fully 

Basis initially completely from artificial variables

When no initial pivots can be derived, this module is used. It is slightly quicker than the partially artificial tableau kind.

Structs

Fully
All variables are artificial.