Enum relp::data::linear_program::elements::BoundDirection [−][src]
pub enum BoundDirection {
Lower,
Upper,
}Expand description
Direction of a bound.
Is used more generally in the case where the three variants of the ConstraintType don’t suit
the needs.
Variants
In the case of a variable, x >= b >= 0 (currently, variables are rewritten to be nonnegative and branching might only bring that bound higher).
In the case of a variable, 0 <= x <= b.