Scaling

Struct Scaling 

pub struct Scaling<R> { /* private fields */ }
Scaling for a linear program.

Describes how the coefficients of the cost function, the rows of the constraints and the variables should be scaled.

Note that the coefficients of the constraint are influenced by both the values applying to the constraint rows and the values applying to the constraint columns.

To invert the scaling, apply the opposite scaling operation to each element.

Implementations

Source

impl<T> Scaling<T>

pub fn scale_back<S>(&self, vector: &mut SparseVector<S, S>)
where for<'r> S: MulAssign<&'r T> + SparseElement<S> + SparseComparator,

Scale a solution vector back.

When a solution is computed using a scaled linear program, that solution is of course also a scaled version of the true solution. This method scales the computed solution to compensate for this scaling.