Bound

Struct Bound 

pub struct Bound<F> {
    pub name: String,
    pub values: Vec<SparseTuple<BoundType<F>>>,
}
Specifies a bound on a variable. The variable can either be continuous or integer, while the bound can have any direction.

Fields

name: String

Name of the bound. Stored only for writing the problem to disk.

values: Vec<SparseTuple<BoundType<F>>>

Collection of at most one bound per row. Note that across different bounds, multiple values per row may be specified.